--
--
-- Last Modified: 2023-01-09 21:43:25
--
-- CREATE DATABASE myhouse;
-- \c myhouse

-- /usr/share/postgresql-common/pgdg/apt.postgresql.org.sh
-- echo "deb https://packagecloud.io/timescale/timescaledb/debian/ $(lsb_release -c -s) main" | sudo tee /etc/apt/sources.list.d/timescaledb.list
-- wget --quiet -O - https://packagecloud.io/timescale/timescaledb/gpgkey | sudo apt-key add -
-- apt-get update && apt-get install timescaledb-2-postgresql-XX
-- su -
-- exec bash
-- timescaledb-tune
-- systemctl restart postgresql

CREATE EXTENSION IF NOT EXISTS timescaledb;

DROP TABLE option;
DROP SEQUENCE option_seq;
CREATE SEQUENCE option_seq;
CREATE TABLE option (
    opt_id bigint NOT NULL DEFAULT nextval('option_seq'),
    opt_option varchar(64) NOT NULL,
    opt_value varchar(256) NOT NULL,
    CONSTRAINT options_idx1 PRIMARY KEY(opt_id)
);

INSERT INTO OPTION (opt_option,opt_value) VALUES
    ('energy_reset','monthly')
    ,('log_level','3'),
    ,('panel_sensors','11,10'),
    ,('device_path','/dev/ttyUSB0');

DROP TABLE relay;
CREATE TABLE relay (
    rel_id bigint NOT NULL,
    rel_label varchar(64) NOT NULL,
    rel_activate varchar(16) NOT NULL,
    rel_threshold_on bigint NOT NULL,
    rel_threshold_off bigint NOT NULL,
    rel_override varchar(16) NOT NULL,
    rel_comment text
);

INSERT INTO relay (
    rel_id,rel_label,rel_activate,rel_threshold_on,rel_threshold_off,rel_override,rel_comment
    ) VALUES
     (1,'Pedestal','soft',30,40,'NONE','On grid when turned off')
    ,(2,'E/V Charger','hard',40,70,'NONE','Not connected when turned off')
    ,(3,'Oven / Range','hard',70,90,'NONE','On grid when turned off');

DROP TABLE power_monitor;
CREATE TABLE power_monitor (
    pwm_sensor int4 NOT NULL,
    pwm_stamp timestamp NOT NULL,
    pwm_volts numeric(6, 1) not NULL,
    pwm_amps numeric(7, 4) not NULL,
    pwm_watts numeric(7, 1) not NULL,
    pwm_energy numeric(14, 0) not NULL,
    pwm_frequency numeric(4, 1) not null,
    pwm_power_factor numeric(5, 2) not null,
    pwm_alarm numeric(2, 0) not null
);
CREATE INDEX power_monitor_idx2 ON power_monitor(pwm_stamp,pwm_sensor)

SELECT create_hypertable('power_monitor','pwm_stamp','pwm_sensor');

DROP VIEW pwm_minute_view;
CREATE VIEW pwm_minute_view AS
SELECT pwm_sensor
    ,date_trunc('minute',pwm_stamp) AS pwm_minute
    ,avg(pwm_volts)::numeric(5,2) AS volts
    ,avg(pwm_amps)::numeric(5,2) AS amps
    ,avg(pwm_watts)::numeric(8,2) AS watts
    ,max(pwm_energy)::numeric(10,0) AS energy
    ,avg(pwm_frequency)::numeric(6,4) AS freq
    ,avg(pwm_power_factor)::numeric(5,4) as power_factor
  FROM power_monitor
  GROUP BY pwm_minute, pwm_sensor
  ORDER BY pwm_minute desc, pwm_sensor;
ALTER TABLE pwm_minute_view OWNER TO myhouse;

DROP VIEW pwm_hour_view;
CREATE VIEW pwm_hour_view AS
  SELECT pwm_sensor
  ,date_trunc('hour',pwm_stamp) AS pwm_hour
    ,avg(pwm_volts)::numeric(5,2) AS volts
    ,avg(pwm_amps)::numeric(5,2) AS amps
    ,avg(pwm_watts)::numeric(8,2) AS watts
    ,max(pwm_energy)::numeric(10,0) AS energy
    ,avg(pwm_frequency)::numeric(6,4) AS freq
    ,avg(pwm_power_factor)::numeric(5,4) AS power_factor
  FROM power_monitor
  GROUP BY pwm_hour, pwm_sensor
  ORDER BY pwm_hour desc, pwm_sensor;
ALTER TABLE pwm_hour_view OWNER TO myhouse;

DROP VIEW pwm_hour_amps;
CREATE VIEW pwm_hour_amps AS
  SELECT time_bucket('1 hour',pwm_stamp,'30 minutes'::interval) + '30 minutes' as pwm_hour
    ,max(pwm_amps)::numeric(5,2) AS amps
  FROM power_monitor
  GROUP BY pwm_hour
  ORDER BY pwm_hour desc;
ALTER TABLE pwm_hour_view OWNER TO myhouse;

DROP VIEW pwm_daily_view;
CREATE VIEW pwm_daily_view AS
  SELECT pwm_sensor
  ,date_trunc('day',pwm_stamp) AS pwm_day
    ,avg(pwm_volts)::numeric(5,2) AS volts
    ,avg(pwm_amps)::numeric(5,2) AS amps
    ,avg(pwm_watts)::numeric(8,2) AS watts
    ,max(pwm_energy)::numeric(10,0) AS energy
    ,avg(pwm_frequency)::numeric(6,4) AS freq
    ,avg(pwm_power_factor)::numeric(5,4) AS power_factor
  FROM power_monitor
  GROUP BY pwm_day, pwm_sensor
  ORDER BY pwm_day desc, pwm_sensor;
ALTER TABLE pwm_daily_view OWNER TO myhouse;
