//
//        Source: hidrelay.h
//         Title: Include file for hidrelay library
//
// Copyright (C) 2022-2023, Keith Edwin Smith
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of Keith Edwin Smith nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY Keith Edwin Smith ''AS IS'' AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Keith Edwin Smith BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Last Modified: 2023-01-03 10:39:13
//

#include <stdio.h>
#include <wchar.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdint.h>
#include <hidapi/hidapi.h>

#define MAX_RELAYS  32

#define HIDRELAY_ON 0xff
#define HIDRELAY_OFF 0xfd
#define HIDRELAY_QUERY 0x01 /* Internal Use */

#define DEFAULT_USB_ID "16c0:05df"
#define HIDRELAY_SHOW_STATE 0
#define HIDRELAY_SHOW_REQUEST 1

union hidrelay_command {
    struct relay_command_struct {
        char report;
        char state;
        char relay;
        char fill[6];
    } c;
    struct relay_name_struct {
        char report;
        char command;
        char name[5];
        char fill[2];
    } n;
    struct relay_result_struct {
        char name[7];
        uint8_t result[2];  // For now assume  we
    } r;
};
typedef union hidrelay_command HIDRELAY_COMMAND;

#define HIDRELAY_LAST_ERROR_SIZE 256
#define HIDRELAY_FLAG_KEEP_OPEN 0x02
#define HIDRELAY_FLAG_VERBOSE 0x01
struct hidrelay_struct {
    char *name;
    char *device_path;
    hid_device *hd;
    int relay_count;
    int state[MAX_RELAYS];
    int request[MAX_RELAYS];
    char *last_error[MAX_RELAYS];
    struct hidrelay_struct *next;
};
typedef struct hidrelay_struct HIDRELAY;


HIDRELAY *hidrelay_fetch(HIDRELAY *hr, char *device_path, int relay_count, unsigned int flags);
int hidrelay_set(HIDRELAY *hr);
HIDRELAY *hidrelay_find(unsigned int flags);
int hidrelay_name(HIDRELAY *hr, char *name);
int hidrelay_show(HIDRELAY *hr,int flag);
char *hidrelay_status(unsigned char status);
