#include "hidrelay.h"

// Get the status of all the relays
HIDRELAY *hidrelay_fetch(HIDRELAY *hr, char *device_path, int relay_count, unsigned int flags) {
    int ix;
    hid_device *hd;
    HIDRELAY_COMMAND hc;

    // Were we sent a relay structure?
    if(hr != NULL) {
        // Were we sent a device path along with it?
        if(device_path != NULL) {
            // And did the device name change?
            if(strcmp(device_path,hr->device_path) != 0) {
                fprintf(stderr,"Device Change %s -> %s\n",hr->device_path, device_path);
                return(NULL); // For now an error
            }
        }
        if(hr->hd != NULL) {
            hd = hr->hd;
        } else {
            hd = hid_open_path(hr->device_path);
        }
        if(hd == NULL) {
            if(flags & HIDRELAY_FLAG_VERBOSE) {
                fprintf(stderr,"unable to open device: %s\n",device_path);
            }
            return(NULL);
        }
    } else {
        hd = hid_open_path(device_path);
        if(hd == NULL) {
            if(flags & HIDRELAY_FLAG_VERBOSE) {
                fprintf(stderr,"unable to open device: %s\n",device_path);
            }
            return(NULL);
        }
        hr = malloc(sizeof(HIDRELAY));
        memset(hr,0,sizeof(HIDRELAY));
        hr->device_path = strdup(device_path);
        if(flags & HIDRELAY_FLAG_KEEP_OPEN) {
            hr->hd = hd;
        }
        hr->relay_count = relay_count;
    }

    // Okay zero the command structure and get the report
    memset(&hc,0,sizeof(hc));
    hc.c.report = 1;  // Feature report 1
    ix = hid_get_feature_report(hd,(unsigned char *)&hc,sizeof(hc));
    if(ix == -1) {
        fprintf(stderr,"%s\n",(char *)hid_error(hd));
        hid_close(hd);
        return(NULL);
    }

    // Now parse the result
    hr->name = strdup(hc.r.name);
    // unsigned char -- 8 bits in result[0]
    for(ix = 0; ix < 8; ix++) {
        hr->state[ix] = (hc.r.result[0] & (1 << ix)) != 0 ? HIDRELAY_ON : HIDRELAY_OFF;
    }
    if(hr->hd == NULL) {
        hid_close(hd);
    }
    return(hr); // SUCCESS returns new or given structure
}
