//----------------------------------------------------------------------
//
// base64.c
// Encode/Decode base64 strings
//
//   Modification History:
// 2004-03-25 Keith Edwin Smith
// o Very old bug was not shifting leftover bits at the final encode
//   (EOF) caused error when not an even 3 boundary, Oops!
//
//----------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
/**/


#define CHARS_PER_LINE 60


#if defined(STANDALONE)

char *argv_flags[] = {
    "-d",
    "-e",
    "--help",
    "-h",
    NULL
};

char *u_message = "\n"
"\n"
"usage: base64 {-h|--help}\n"
"       base64 {-d|-e} [infile] [outfile]\n"
"       base64_encode [infile] [outfile]\n"
"       base64_decode [infile] [outfile]\n"
"\n"
"    -h --help -- Print this text and exit"
"    -d decode -- convert base-64 encoded file back to normal\n"
"    -e encode -- convert file into a base-64 encoded file\n"
"\n"
"    If either infile or outfile are missing input or output are\n"
"    directed to stdin and stdout as appropriate.\n"
"    Specifying infile or outfile as a '-' will force i/o onto the\n"
"    stdin/stdout channels\n"
"\n"
"\n";

usage(pgm_name)
char *pgm_name;
{
    fprintf(stderr,u_message,pgm_name);
}

main(argc, argv)
int argc;
char *argv[];
{
    FILE *in, *out;
    char out_name[1024];
    char in_name[1024];
    int flag, i, x;

    in = stdin ;
    out = stdout ;
    flag = in_name[0] = out_name[0] = 0;
    if(strcmp(argv[0],"base64_encode") == 0) flag = 1;
    if(strcmp(argv[0],"base64_decode") == 0) flag = 2;
    for ( i = 1 ; i < argc ; i++ ) {
        x = argvindex(argv_flags, argv[i], strcmp);
        switch(x) {
        case 0:
            flag = 2;
            continue;
        case 1:
            flag = 1;
            continue;
        case 2:
        case 3:
            usage(argv[0]);
            exit(0);
        default:
            if(in_name[0] == 0) {
                strcpy(in_name, argv[i]);
            } else if(out_name[0] == 0) {
                strcpy(out_name, argv[i]);
            } else {
                usage(argv[0]);
                exit(3);
            }
        } /* switch */
    } /* for */
    if(in_name[0] != 0) {
        in = fopen(in_name, "r");
        if (in == NULL) {
            fprintf(stderr, "Could not open %s\n", in_name);
            exit(1);
        }
    }
    if(out_name[0] != 0) {
        out = fopen(out_name, "w");
        if (out == NULL) {
            fprintf(stderr, "Could not open %s\n", out_name);
            exit(2);
        }
    }
    switch(flag) {
    case 2:
        base64_decode(in, out);
        break;
    case 1:
        base64_encode(in, out);
        break;
    case 0:
        usage(argv[0]);
        exit(5);
    }
    if(in != stdin) fclose(in);
    if(out != stdout) fclose(out);
}
#endif

int base64_decode(in, out)
FILE *in, *out;
{
    unsigned char aline[120];
    unsigned int c1,w1,wx;
    int b1,cv[128];
    int i,ip,j;

 /* Initialize table */
    for(i=0;i<128;i++) cv[i] = -1;
    for(i=0;i<26;i++) {
        cv[i+'A'] = i;
        cv[i+'a'] = 26+i;
    }
    for(i=0;i<10;i++) cv[i+'0'] = 52+i;
    cv['+'] = 62;
    cv['/'] = 63;
    cv['='] = -2;  /* padd character */
    ip = 0;
    w1 = 0;

    while(fgets(aline, 120, in) != NULL) {
        for(j=0;j<strlen(aline)-1;j++) {
            c1 = aline[j];
            b1 = cv[c1];
            if(b1 >= 0) {
                w1 = (w1 << 6) | b1;
                ip += 6;
                if(ip>=8) {
                    wx = w1 >> (ip - 8);
                    ip -= 8;
                    wx &= 255;
                    putc(wx, out);
                    w1 &= 0xffff;
                }
            }
        }
    }
}

int base64_encode(in, out)
FILE *in, *out;
{
    unsigned char aline[120];
    unsigned int w_bits, t_bits;
    int b64_etab[128];
    int i, ix, cc, bits_waiting;

    ix = 0;
 /* Initialize encoding table */
    for(i=0;i<26;i++) {
        b64_etab[i] = 'A'+ i;
        b64_etab[i+26] = 'a' + i;
    }
    for(i = 0; i < 10; i++) b64_etab[52 + i] = '0' + i;
    b64_etab[62] = '+';
    b64_etab[63] = '/';
    b64_etab[64] = '=';

    t_bits = w_bits = bits_waiting = 0;
    while( (cc = fgetc(in)) != EOF ) {
        w_bits = (w_bits << 8 ) | (cc & 0xff);  /* get 8 more bits */
        bits_waiting += 8;
        while(bits_waiting > 6) {
            t_bits = w_bits >> (bits_waiting - 6); /* Pick off top 6 */
            aline[ix++] = b64_etab[t_bits & 0x3f];
            if(ix > CHARS_PER_LINE) {
                aline[ix] = 0;
                fprintf(out, "%s\n", aline);
                ix = 0;
            }
            bits_waiting -= 6;
            cc = (1 << bits_waiting) -1 ;
            w_bits = w_bits & cc;
        }
    }
    if(bits_waiting) {
        t_bits = w_bits << (6 - bits_waiting);
        aline[ix++] = b64_etab[t_bits & 0x3f];
    }
    if(ix) {
        for(i=0; i< (ix % 4); i++) aline[ix++] = '=';
        aline[ix] = 0;
        fprintf(out, "%s\n", aline);
    }
}
