//
//
//        Source: config.c
//         Title: INI style configuration file functions
//       Created: Thu Dec 18 13:57:54 2022
//            By: Keith Edwin Smith, Casa Fiesta Network Servics
//     Copyright: (c)2022 Keith Edwin Smith
//
// Last Modified: 2023-01-04 20:59:34
//
// NOTE: This was added to libax recently and is based on a
//       PHP library I built moons ago
//
// TODO: Pretty much everything.
//
#include "local.h"
#include <string.h>
#include <stdlib.h>

// Something as a limit.
#define CONFIG_BUF_LEN  4096

//----------------------------------------------------------------------
// __CONFIG_PARSE_ENTRY
//----------------------------------------------------------------------
CONFIG_FILE_ENTRY *__config_parse_entry(buffer)
char *buffer;
{
    CONFIG_FILE_ENTRY *ce;
    int x;

    ce = (CONFIG_FILE_ENTRY *) malloc(sizeof(CONFIG_FILE_ENTRY));
    memset(ce,0,sizeof(CONFIG_FILE_ENTRY));

    x = strcpos(buffer, '=');
    if(*buffer == 0 || *buffer == ';' || *buffer == '#' || x < 0) {
        ce->key = malloc(1);
        ce->key[0] = 0;
        ce->value = malloc(strlen(buffer) + 1);
        strcpy(ce->value, buffer + x + 1);
        strcvs(ce->value, CVS_LEADING | CVS_TRAILING);
        return(ce);
    }

    ce->key = malloc(x + 1);
    memset(ce->key,0,x + 1);
    strncpy(ce->key, buffer, x);
    strcvs(ce->key, CVS_LEADING | CVS_TRAILING);

    ce->value = malloc( (strlen(buffer) - x) + 2);
    memset(ce->value,0,(strlen(buffer) - x) + 2);
    strcpy(ce->value, buffer + x + 1);
    strcvs(ce->value, CVS_LEADING | CVS_TRAILING);

    ce->next = NULL;

    return(ce);
}

//----------------------------------------------------------------------
// CONFIG_FREE
// Free resource in a config tree
//----------------------------------------------------------------------
int config_free(CONFIG_FILE_SECTION *first) {
    CONFIG_FILE_SECTION *cs,*cs_next;
    CONFIG_FILE_ENTRY *ce,*ce_next;

    for(cs = first; cs != NULL; cs = cs_next) {
        cs_next = cs->next;
        if(cs->name != NULL) free(cs->name);
        for(ce = cs->entry; ce != NULL ; ce = ce->next) {
            ce_next = ce->next;
            if(ce->key != NULL) free(ce->key);
            if(ce->value != NULL) free(ce->value);
            free(ce);
        }
    }
    return(1);
}

//----------------------------------------------------------------------
// CONFIG_READ_FILE
//----------------------------------------------------------------------
CONFIG_FILE_SECTION *config_read_file(char *name, unsigned int flags)
{
    FILE *fconfig;
    char buffer[CONFIG_BUF_LEN];
    CONFIG_FILE_SECTION *cs, *cs_first;
    CONFIG_FILE_ENTRY *ce;
    int l,ix;
    char *p;

    // Sanity
    memset(buffer,0,CONFIG_BUF_LEN);
    // Abort if we cannot open the file
    fconfig = fopen(name,"r");
    if(fconfig == NULL) {
        return(NULL);
    }

    // Global Section, name is blank
    cs_first = (CONFIG_FILE_SECTION *) malloc(sizeof(CONFIG_FILE_SECTION));
    cs = cs_first;
    memset(cs,0,sizeof(CONFIG_FILE_SECTION));
    cs->name = "";
    cs->entry = NULL;
    cs->next = NULL;
    for(;;) {
        if( (l = readline(fconfig, buffer, CONFIG_BUF_LEN)) < 0 ) {
            break;
        }

        strcvs(buffer, CVS_LEADING | CVS_TRAILING | CVS_MULTIPLE);
        p = buffer;
        if(*p == '[') { /* New section */
            cs->next = (CONFIG_FILE_SECTION *) malloc(sizeof(CONFIG_FILE_SECTION));
            cs = cs->next;
            memset(cs,0,sizeof(CONFIG_FILE_SECTION));
            cs->next = NULL;
            p++;
            ix = strcpos(p,']');
            // Check failure ... -1 I think
            p[ix] = 0;
            cs->name = strdup(p);
            cs->entry = NULL;
            cs->next = NULL;
        } else { // Key/Value item or comment line
            if(cs->entry == NULL) {
                cs->entry = __config_parse_entry(buffer);
                ce = cs->entry;
            } else {
                ce->next = __config_parse_entry(buffer);
                ce = ce->next;
            }
        }
    }
    return(cs_first);
}

//----------------------------------------------------------------------
// CONFIG_SECTION
// Find a value in a section
//----------------------------------------------------------------------
CONFIG_FILE_SECTION *config_section(CONFIG_FILE_SECTION *first, char *section) {
    CONFIG_FILE_SECTION *cs;

    for(cs = first; cs != NULL; cs = cs->next) {
        if(strcmp(cs->name,section) == 0) {
            return(cs);
        }
    }
    return(NULL);
}

//----------------------------------------------------------------------
// CONFIG_VALUE
// Find a value in a section
//----------------------------------------------------------------------
char *config_value(CONFIG_FILE_SECTION *first, char *section, char *key) {
    CONFIG_FILE_SECTION *cs;
    CONFIG_FILE_ENTRY *ce;

    for(cs = first; cs != NULL; cs = cs->next) {
        if(strcmp(cs->name,section) == 0) {
            for(ce = cs->entry; ce != NULL ; ce = ce->next) {
                if(strcmp(ce->key,key) == 0) {
                    return(ce->value);
                }
            }
            return(NULL);
        }
    }
    return(NULL);
}

//----------------------------------------------------------------------
// CONFIG_WRITE_FILE
//----------------------------------------------------------------------
int config_write_file(char *path, CONFIG_FILE_SECTION *first, unsigned int flags) {
    FILE *fp;
    CONFIG_FILE_SECTION *cs;
    CONFIG_FILE_ENTRY *ce;

    fp = fopen(path,"w");
    for(cs = first; cs != NULL; cs = cs->next) {
        if(strcmp(cs->name,"") != 0) {
            fprintf(fp,"[%s]\n",cs->name);
        }
        if(cs->entry == NULL) {
            continue;
        }
        for(ce = cs->entry; ce != NULL; ce = ce->next) {
            if(strcmp(ce->key,"") == 0) {
                fprintf(fp,"%s\n",ce->value);
            } else {
                fprintf(fp,"%s = %s\n",ce->key,ce->value);
            }
        }
    }
    fclose(fp);
}
