/*
	local.h - Function Prototypes for local library functions
*/

#ifndef _LOCAL_H
#define _LOCAL_H

/* strcvs func args */
#define CVS_LEADING         1
#define CVS_TRAILING        2
#define CVS_UPPER           4
#define CVS_LOWER           8
#define CVS_NON_PRINTABLE   16
#define CVS_MULTIPLE        32

/* config_* func args */
#define CFG_FLAG_QUOTED     1

#ifndef _STDIO_H
#include <stdio.h>
#endif
#include <sys/types.h>

//
// config.c
//
struct config_file_entry_struct {
    char *key;
    char *value;
    struct config_file_entry_struct *next;
};
typedef struct config_file_entry_struct CONFIG_FILE_ENTRY;

struct config_file_section_struct {
    char *name;
    CONFIG_FILE_ENTRY *entry;
    struct config_file_section_struct *next;
};
typedef struct config_file_section_struct CONFIG_FILE_SECTION;

//
// log.c
//
#define LOG_STATUS_NEWLINE_PENDING 0x01
#define LOG_FOLDER "/var/log/USER"
#define LOG_TO_STDERR "*STDERR"
#define LOG_TO_STDOUT "*STDOUT"
#define LOG_TO_NULL "*NULL"
struct log_struct {
    FILE *fp;
    char *path;
    int log_level;
    unsigned int status;
};
typedef struct log_struct LOG_STRUCT;


#ifndef NO_PROTOTYPE

int     argvindex(char *argvlist[], char *s, int (*strfunct)());

// From config.c
int config_free(CONFIG_FILE_SECTION *first);
CONFIG_FILE_SECTION *config_read_file(char *name, unsigned int flags);
CONFIG_FILE_SECTION *config_section(CONFIG_FILE_SECTION *cfg, char *section);
char *config_value(CONFIG_FILE_SECTION *cfg, char *section, char *key);
int config_write_file(char *path, CONFIG_FILE_SECTION *first, unsigned int flags);
// --

int	expand(char *target, char *source, int length);
int	fdreadln(int fd, char *linebuf, char *fdbuf, int *bptr,
		int *bsize, int limit);
LOG_STRUCT *log_open(char *name, int level, LOG_STRUCT *log);
int     log_print(LOG_STRUCT *log, int level, char *format, ...);
int     makepath(char *path, mode_t mode);
int	memicmp(char *aa, char *bb, int n);
int	readline(FILE *f, char *buffer, int length);
long	scanback(FILE *f, long lines);
char    **strargs(char *s);
int	strcpos(const char *aa, int cc);
int	strrcpos(const char *aa, int cc);
int     strcvs(char *aa, int func);
int	stricmp(char *aa, char *bb);
int	strincmp(char *aa, char *bb, int n);
int	strpack(char *s);
int	strupr(char *aa);
int	strlwr(char *aa);
int	substr(char *p, char *q);
int     tcp_open(char *host, char *port);

#endif

#endif /* _LOCAL_H */
