//
//
// Last Modified: 2023-01-08 08:21:06
//

#include <local.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <sys/stat.h>
#include <stdarg.h>
#include <unistd.h>

//----------------------------------------------------------------------
// LOG_OPEN
//----------------------------------------------------------------------
LOG_STRUCT *log_open(char *name, int level, LOG_STRUCT *log) {
    char buf[4096];
    char ftime[64];
    struct tm *tm_p;
    time_t curtime;
    struct stat sb;

    curtime = time(NULL);
    tm_p = localtime(&curtime);
    strftime(ftime,63,"%Y%m%d",tm_p);
    sprintf(buf,"%s/%s/%s",LOG_FOLDER,name,ftime);

    // Allocate if new
    if(log == NULL) {
        log = malloc(sizeof(LOG_STRUCT));
        memset(log,0,sizeof(LOG_STRUCT));
    }

    // Level change only, but set stderr if no output yet
    if(name == NULL) {
        if(log->fp != NULL) {
            log->fp = stderr;
            log->path = strdup(LOG_TO_STDERR);
        }
        log->log_level = level;
        return(log);
    }
    // Close if the path changed
    if(log->fp != NULL) {
        if(strcmp(log->path,buf) != 0) {
            if(!(strcmp(log->path,LOG_TO_STDERR) == 0) || (strcmp(log->path,LOG_TO_STDOUT) == 0)) {
                fclose(log->fp);
            }
            log->fp = NULL;
            free(log->path);
        }
    }
    // Re-open if new or closed
    if(log->fp == NULL) {
        // UGLY
        if(strcmp(name,LOG_TO_STDERR) == 0) {
            log->fp = stderr;
            log->path = strdup(name);
        } else if(strcmp(name,LOG_TO_STDOUT) == 0) {
            log->fp = stdout;
            log->path = strdup(name);
        } else if(strcmp(name,LOG_TO_NULL) == 0) {
            log->path = "/dev/null";
            log->fp = fopen(log->path,"a");
        } else {
            // Build the sub-folder if it isn't there
            sprintf(buf,"%s/%s",LOG_FOLDER,name);
            if(stat(buf,&sb)) {
                if(!(sb.st_mode & S_IFDIR)) {
                    mkdir(buf,0777);
                    chmod(buf,0777);
                }
            }
            sprintf(buf,"%s/%s/%s",LOG_FOLDER,name,ftime);
            log->fp = fopen(buf,"a");
            log->path = strdup(buf);
        }
        setvbuf(log->fp, (char *)NULL, _IOLBF, 0);
    }
    log->log_level = level;
    return(log);
 }

//----------------------------------------------------------------------
// LOG_PRINT
//----------------------------------------------------------------------
int log_print(LOG_STRUCT *log, int level, char *format, ...)
{
    va_list args;
    char ftime[64],*p;
    struct tm *tm_p;
    time_t curtime;

    va_start(args,format);
    if(log->log_level >= level) {
        curtime = time(NULL);
        tm_p = localtime(&curtime);
        strftime(ftime,63,"%H:%M:%S",tm_p);
        p = malloc(strlen(ftime) + strlen(format) + 32);
        if(log->status & LOG_STATUS_NEWLINE_PENDING) {
            strcpy(p,format);
        } else {
            sprintf(p,"%s (%d) %s",ftime,getpid(),format);
        }
        if(p[strlen(p) - 1] != '\n') {
            log->status |= LOG_STATUS_NEWLINE_PENDING;
        } else {
            log->status &= ~LOG_STATUS_NEWLINE_PENDING;
        }
        curtime = time(NULL);
        vfprintf(log->fp,p,args);
    }
    va_end(args);
}
