//----------------------------------------------------------------------
// tcp_open.c
//
// Created By: Keith Edwin Smith
//    Created: Mon Nov 14 08:01:22 2005
//
// Copyright (C) 2005 Keith Edwin Smith
// All Rights Reserved.
// This  document  may  not, in  whole  or in  part, be  copied,  photocopied,
// reproduced,  translated,  or  reduced to any  electronic  medium or machine
// readable form without prior written consent from Keith Edwin Smith.
//
// Last Modified: 2005-11-14 08:38:41
//
// Modification History:
//----------------------------------------------------------------------
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>


int tcp_open(host, port)
char *host;
char *port;
{
    struct hostent *hp;
    struct servent *sp;
    static struct sockaddr_in adrp;
    int i, skt, rval, service;
    long laddr;
    struct linger stay;


    if(host == NULL) {
        fprintf(stderr,"Host is NULL in tcp_open call\n");
        return(-1);
    }
    service = atoi(port);
    if(service != 0) {
        sp = (struct servent *)malloc(sizeof(struct servent));
        sp->s_port = htons(service);
        sp->s_name = strdup(port);
        sp->s_proto = strdup("tcp");
    } else {
        sp = getservbyname(port, "tcp");
        service = ntohs(sp->s_port);
    }
    if(sp == NULL) {
        fprintf(stderr, "Could not find port named %s.\n", port);
        return(-4);
    }
    if(service < 1 || service > 65535) {
        fprintf(stderr,"Bad Port %d\n", service);
    }
    hp = gethostbyname(host);
    if(hp == NULL) {
        fprintf(stderr, "Could not resolve hostname %s.\n", host);
        return(-3);
    }
#if DEBUG > 8
    fprintf(stderr, "got service (%s) = %d \n", sp->s_name, ntohs(sp->s_port) );
    fprintf(stderr, "got service proto = %s \n", sp->s_proto);
    fprintf(stderr, "got host = %s\n", hp->h_name);
    fprintf(stderr, "got length = %d\n", hp->h_length);
    fprintf(stderr, "got addr = %s\n", hp->h_addr_list[0]);
    fprintf(stderr, "got_addr = ");
    for ( i = 0 ; i < hp->h_length ; i++ ) {
        fprintf(stderr, "%d ", (unsigned char)hp->h_addr[i]);
    }
    fprintf(stderr, "\n");
#endif

    skt = socket(AF_INET, SOCK_STREAM, 0);
    if ( skt < 0 ) {
        fprintf (stderr, "Socket returns %d\n", skt);
        return(-2);
    }
    adrp.sin_family = AF_INET;
    adrp.sin_port = (short)sp->s_port;
    memcpy(&adrp.sin_addr.s_addr, hp->h_addr, (size_t)hp->h_length);

#if 1
    stay.l_onoff = 0;
    stay.l_linger = 0;
    rval = setsockopt (skt, SOL_SOCKET, SO_LINGER, &stay, sizeof(stay));
#endif

    rval = connect(skt, (struct sockaddr *)&adrp, sizeof(adrp) );
    if ( rval != 0 ) {
        perror("connect");
        return(-1);
    }

#if DEBUG > 9
    fprintf (stderr, "Connect returns %d\n", rval);
#endif

    return(skt);
}


#if 0
main(argc, argv)
int argc;
char *argv[];
{
    int fd_in, fd_out;
    FILE *fin, *fout;
    char buffer[256];

    fd_in = tcp_open("ksmith.com","pop3");
    fd_out = dup(fd_in);
    fin = fdopen(fd_in, "r");
    setvbuf(fin, (char *)NULL, _IOLBF, 0);
    fout = fdopen(fd_out, "w");
    setvbuf(fout, (char *)NULL, _IOLBF, 0);
    fgets(buffer, 255, fin);
    fprintf(stderr,"buffer = %s\n", buffer);
    fputs("user keith\r\n", fout);
    fgets(buffer, 255, fin);
    fprintf(stderr,"buffer = %s\n", buffer);
}

#endif
