//
//
//        Source: add_table_entry.c
//         Title: Add polling data to our sql table
//    Created By: Keith Edwin Smith
// Last Modified: 2023-01-08 15:25:33
//       Created: Sun Jan 8 10:10:05 2023
//
// Copyright (C) 2022-2023, Keith Edwin Smith
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of Keith Edwin Smith nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Modification History:
//

#include "panelmonitor.h"

//----------------------------------------------------------------------
// ADD_TABLE_ENTRY
//----------------------------------------------------------------------
int add_table_entry(PGconn *myhouse, struct sensor_values *sv_p) {
    char sql[2048];
    PGresult *res;

    if(sv_p->amps == 0) {
        if(g.verbose) {
            fprintf(stderr,"No Current value for poll, sensor = %d\n",sv_p->sensor_id);
        } else {
            log_print(g.log,1,"ERROR: Current = 0, sensor = %d\n",sv_p->sensor_id);
        }
        return(0);
    }
    sprintf(sql,"INSERT INTO power_monitor\n"
            "  (\n"
            "    pwm_sensor,pwm_stamp\n"
            "    ,pwm_volts,pwm_amps,pwm_watts,pwm_energy\n"
            "    ,pwm_frequency,pwm_power_factor,pwm_alarm\n"
            "  ) VALUES (\n"
            "    %d,'%s'\n"
            "    ,%0.1f,%0.3f,%0.1f,%ld\n"
            "    ,%0.1f,%0.2f,%d\n"
            "  )\n"
            ,sv_p->sensor_id,sv_p->stamp
            ,sv_p->volts,sv_p->amps,sv_p->watts,sv_p->energy
            ,sv_p->frequency,sv_p->power_factor,sv_p->alarm
            );

    if(g.verbose) {
        fprintf(stdout,"--\n");
        fprintf(stdout,"%s",sql);
        fprintf(stdout,"--\n");
    }

    res = PQexec(myhouse, sql);
    if (PQresultStatus(res) != PGRES_COMMAND_OK)  {
        log_print(g.log,0,"Insert Failed %s\n",
                PQerrorMessage(myhouse)
                  );
        log_print(g.log,0,"%s\n",sql);
        // Try a reset, and quit if that doesn't help
        PQreset(myhouse);
        res = PQexec(myhouse, sql);
        if (PQresultStatus(res) != PGRES_COMMAND_OK)  {
            log_print(g.log,0,"Insert Failed after reset %s\n",
                      PQerrorMessage(myhouse)
                      );
            end_program();
        }
    }
    PQclear(res);
    return(0);
}
