//
//        Source: check_periodic.c
//         Title: Called after every poll for time dependencies
//    Created By: Keith Edwin Smith
// Last Modified: 2023-01-10 12:24:55
//       Created: Mon Jan 2 19:36:12 2023
//
// Copyright (C) 2022-2023, Keith Edwin Smith
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of Keith Edwin Smith nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Modification History:

#include "panelmonitor.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>


// Read the relays every 5 minutes
#define CHECK_RELAY_INTERVAL 300

struct tm *check_periodic() {
    time_t t;
    struct tm *now;
    static struct tm *last = NULL;
    static char buf[32];
    static int last_relay_check = 0;
    struct stat statbuf;
    int err;

    HIDRELAY *hr;

    if(last == NULL) {
        last = malloc(sizeof(struct tm));
        memset(last,0,sizeof(struct tm));
    }
    t = time(NULL);
    now = localtime(&t);
    strftime(buf,32,"%Y%m%d",now);

    // Re-open log if a new day
    if(strstr(g.log->path,buf) == NULL) {
        g_args.log = g.log = log_open("panelmonitor",g.log_level,g.log);
    }

    // Look for a config file change
    err = stat(CONFIG_FILE,&statbuf);
    if(err == -1) {
        log_print(g.log,0,"Config File Missing\n");
    } else {
        if(statbuf.st_mtime > g.cfg_read_time) {
        log_print(g.log,1,"Config File Update\n");
            parse_config();
            log_relay_status(g.log,1);
        }
    }

    if( (t - last_relay_check) > CHECK_RELAY_INTERVAL ) {
        log_print(g.log,1,"RELAY CHECK: Last Amps = %d\n",g.last_amps);
        // Update the relay status
        for(hr = g.hr_first; hr != NULL; hr = hr->next) {
            hidrelay_fetch(hr,NULL,0,0);
        }
        last_relay_check = t;
        g.last_amps = 0;
    }
    // Minutes
    if(last->tm_min != now->tm_min) {
        // Show the current relay status
        if((now->tm_min % 15) == 0) {
            log_relay_status(g.log,1);
        }
        last->tm_min = now->tm_min;
    }
    return(now);
}
