//
//        Source: powermeter.c
//       Created: Tue Dec 20 16:02:25 2022
//            By: Keith Edwin Smith
//     Copyright: (c)2022 Keith Edwin Smith
//         Title: Management/Polling/Logging for Energy Sensor
//
// Last Modified: 2023-01-10 09:15:35
//

#define PANELMONITOR_MAIN 1
#include "panelmonitor.h"

//----------------------------------------------------------------------
// MAIN
//----------------------------------------------------------------------
int main(int argc, char *argv[]) {
    int ix,amps;
    char noise_buf[4096],*p;

    // Parse the command line into g_args, save these as priority
    memset(&g_args,0,sizeof(g_args));
    g_args.log = log_open("*STDERR",DEFAULT_LOG_LEVEL,NULL);
    parse_args(argc,argv);

    // Next, read our configuration
    parse_config();
    // Now 'g' is our working global structure
    /* g.hr_first = hidrelay_find(HIDRELAY_FLAG_KEEP_OPEN); Keep open */
    g.hr_first = hidrelay_find(0);
    if(g.hr_first == NULL) {
        log_print(g.log,0,"ERROR: Failed find relays\n");
    }
    g.dbh = db_open(g.cfg);

    // Set up an RS485 sensor handle
    sensor_create();
    switch(g.action) {
    case ACTION_DAEMON:
        if(!g.verbose) {
            g.log = log_open("panelmonitor",g.log_level,g.log);
        }
        // Create a loggable startup line
        p = noise_buf;
        *p = 0;
        strcat(p,"DAEMON:");
        for(ix = 1; ix < argc; ix++) {
            strcat(p," ");
            strcat(p,argv[ix]);
        }
        g.cmd_args = strdup(p);
        log_print(g.log,1,"%s\n",noise_buf);
        show_config(g.log,1);
        for(;;) {
            amps = sensor_poll_all(g.psensor,g.dbh);
            g.last_amps = amps > g.last_amps ? amps : g.last_amps;
            sleep(g.interval);
            check_periodic(); // Check for timeouts/resets
        }
        break;
    case ACTION_POLL_ONCE:
        g.last_amps = sensor_poll_all(g.psensor,g.dbh);
        break;
    case ACTION_RESET:
        for(ix = 0; g.sensor_list[ix] != 0; ix++) {
            sensor_reset_energy(g.psensor, g.sensor_list[ix]);
        }
        break;
    case ACTION_SET_ID:
        sensor_set_id(g.psensor,1,g.new_id);
        break;
    case ACTION_CALIBRATE:
        sensor_calibrate(g.psensor,g.sensor_list[ix]);
        break;
    case ACTION_SHOW:
        show_config(NULL,0);
        break;
    }
    end_program();
}
