//
//
//        Source: parse_config.c
//         Title: Parse configuration object for missing config
//    Created By: Keith Edwin Smith
// Last Modified: 2023-01-08 10:23:58
//       Created: Wed Jan 4 20:27:40 2023
//
// Copyright (C) 2022-2023, Keith Edwin Smith
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of Keith Edwin Smith nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
#include "panelmonitor.h"

int parse_config() {
    char *sensor_ids,**slist,*tmp;
    int ix;
    struct tm *now;
    time_t t;

    log_print(g_args.log,3,"parse_config()\n");
    // Copy the command line arguments
    memcpy(&g,&g_args,sizeof(g));

    // Now Check configuration for undefined things
    if(g.cfg != NULL) {
        log_print(g_args.log,3,"Calling config_free()\n");
        config_free(g.cfg);
        log_print(g_args.log,3,"Call config_free() completed\n");
    }

    // Did we specify a config file?
    if(g.config_path == NULL) {
        g.config_path = CONFIG_FILE;
    }
    g.cfg = config_read_file(g.config_path,0);

    tmp = config_value(g.cfg,"","log_level");
    if(tmp != NULL) {
        g.log_level = strtol(tmp,NULL,0);
        log_open(NULL,g.log_level,g.log);
    }
    log_print(g_args.log,7,"g.cfg address is %p\n",g.cfg);
    g.cfg_read_time = time(NULL);
    if(g.sensor_list[0] == 0) {
        sensor_ids = config_value(g.cfg,"","sensor_ids");
        if(sensor_ids == NULL) {
            fprintf(stderr,"No sensors defined to poll\n");
        } else {
            slist = strargs(sensor_ids);
            for(ix = 0; slist[ix] != NULL; ix++) {
                g.sensor_list[ix] = strtol(slist[ix],NULL,0);
            }
        }
    }
    if(g.device_path == NULL) {
        g.device_path = config_value(g.cfg,"","device_path");
    }
    if(g.energy_reset == NULL) {
        g.energy_reset = config_value(g.cfg,"","energy_reset");
    }
    if(g.energy_reset != NULL) {
        if(argvindex(energy_reset_list,g.energy_reset,strcmp) == RESET_NOW) {
            g.action = ACTION_RESET;
            g.verbose = 1;
        } else {
            t = time(NULL);
            now = localtime(&t);
            reset_check(now,RREQ_INIT,NULL,0);
        }
    }
    // And the relay settings
    parse_config_relay();
}
