//
//
//        Source: parse_config_relay.c
//         Title: Parse the relay section of the config file
//    Created By: Keith Edwin Smith
// Last Modified: 2023-01-08 08:02:37
//       Created: Sun Jan 8 07:57:34 2023
//
// Copyright (C) 2022-2023, Keith Edwin Smith
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of Keith Edwin Smith nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Modification History:
//
//        Source: relay_fetch_config.c
//
// Last Modified: 2023-01-04 21:24:15
//
// Maybe

#define RELAY_DATA 1
#include "panelmonitor.h"

#define RELAY_SECTION "relay"

char *relay_defs[] = {
    "min_off_time"
    ,NULL
};
#define RELAY_MIN_OFF_TIME 0
#define RELAY_NULL 1

char *relay_keys[] = {
    "label"
    ,"threshold_off"
    ,"threshold_on"
    ,"activate"
    ,"override"
    ,NULL
};
#define RELAY_KEY_LABEL 0
#define RELAY_KEY_THRESHOLD_OFF 1
#define RELAY_KEY_THRESHOLD_ON 2
#define RELAY_KEY_ACTIVATE 3
#define RELAY_KEY_OVERRIDE 4
#define RELAY_KEY_NULL 5

//----------------------------------------------------------------------
// PARSE_RELAY_SECTION
//----------------------------------------------------------------------
RELAY *parse_relay_section(char *relay_section) {
    CONFIG_FILE_SECTION *cs;
    CONFIG_FILE_ENTRY *ce;
    RELAY *rs;
    int ix;

    rs = malloc(sizeof(RELAY));
    memset(rs,0,sizeof(RELAY));
    // rs->last_change = time(NULL);

    cs = config_section(g.cfg,relay_section);
    for(ce = cs->entry; ce != NULL ; ce = ce->next) {
        ix = argvindex(relay_keys,ce->key,strcmp);
        switch(ix) {
        case RELAY_KEY_LABEL:
            rs->label = ce->value;
            break;
        case RELAY_KEY_THRESHOLD_OFF:
            rs->threshold_off = atol(ce->value);
            break;
        case RELAY_KEY_THRESHOLD_ON:
            rs->threshold_on = atol(ce->value);
            break;
        case RELAY_KEY_ACTIVATE:
            rs->activate = argvindex(relay_activate_words,ce->value,strcmp);
            break;
        case RELAY_KEY_OVERRIDE:
            rs->override = argvindex(relay_override_words,ce->value,strcmp);
            switch(rs->override) {
            case RELAY_OVERRIDE_ON:
                rs->override = HIDRELAY_ON;
                break;
            case RELAY_OVERRIDE_OFF:
                rs->override = HIDRELAY_OFF;
                break;
            }
            break;
        }
    }
    return(rs);
}

//----------------------------------------------------------------------
// PARSE_CONFIG_RELAY
//----------------------------------------------------------------------
int parse_config_relay() {
    CONFIG_FILE_SECTION *cs,*cs_r;
    CONFIG_FILE_ENTRY *ce,*ce_r;
    int relay_number,ix;
    char *section_name, *val;
    RELAY *rs;

    cs = config_section(g.cfg,RELAY_SECTION);
    if(cs == NULL) {
        fprintf(stderr,"No relays defined\n");
        return(0);
    }
    for(ce = cs->entry; ce != NULL ; ce = ce->next) {
        // Ignore comments and such
        if(ce->key[0] == 0) {
            continue;
        }
        // Relays
        if(strcpos("123456789",ce->key[0]) >= 0) {
            relay_number = atol(ce->key) - 1;
            if(relay_number >= MAX_RELAYS) {
                fprintf(stderr,"Ooops relay_number = %d\n",relay_number);
                exit(-1);
            }
            g.relay[relay_number] = parse_relay_section(ce->value);
        } else {
            ix = argvindex(relay_defs,ce->key,strcmp);
            switch(ix) {
            case RELAY_MIN_OFF_TIME:
                g.relay_min_off_time = atol(ce->value) * 60; // Minutes -> Seconds
                break;
            }
        }
    }
}
