//
//        Source: reset_check.c
//       Created: Tue Dec 20 16:02:25 2022
//            By: Keith Edwin Smith
//     Copyright: (c)2022 Keith Edwin Smith
//         Title: powermeter: Check if a power reset is needed
//
// Last Modified: 2023-01-03 10:57:45
//

#include "panelmonitor.h"

//----------------------------------------------------------------------
// RESET_CHECK
//----------------------------------------------------------------------
int reset_check(struct tm *now, unsigned request, modbus_t *psensor, int sensor_id) {
    int status, frequency, days;
    struct tm next;

    // Create a status based on the requested frequency
    frequency = argvindex(energy_reset_list,g.energy_reset,strcmp);
    status = 0;
    switch(frequency) {
    case RESET_NULL:
        return(0);
    case RESET_DAILY:
        status = g.energy_reset_day == now->tm_mday;
        break;
    case RESET_WEEKLY:
    case RESET_MONTHLY:
        status = (g.energy_reset_month == now->tm_mon && g.energy_reset_day == now->tm_mday);
        break;
    case RESET_YEARLY:
        break;
    }

    // Now check what was requested
    switch(request) {
    case RREQ_NOW: // reset unconditional
        sensor_reset_energy(psensor,sensor_id);
        break;
    case RREQ_STATUS: // reset on status
        if(status) {
            sensor_reset_energy(psensor,sensor_id);
        }
        break;
    case RREQ_UPDATE: // reset on satus & update reset time to next
        if(status) {
            sensor_reset_energy(psensor,sensor_id);
            memcpy(&next,now,sizeof(struct tm));
            switch(frequency) {
            case RESET_DAILY:
                next.tm_mday += 1;
                break;
            case RESET_WEEKLY:
                next.tm_mday += 7;
                break;
            case RESET_MONTHLY:
                next.tm_mon += 1;
                break;
            case RESET_YEARLY:
                next.tm_year += 1;
                break;
            }
            mktime(&next);
            g.energy_reset_month = next.tm_mon;
            g.energy_reset_day = next.tm_mday;
        }
        break;
    case RREQ_INIT: // initialize
        memcpy(&next,now,sizeof(struct tm));
        switch(frequency) {
        case RESET_DAILY:
            next.tm_mday += 1;
            break;
        case RESET_WEEKLY:
            if(g.energy_reset_day < next.tm_wday) {
                days = next.tm_wday - g.energy_reset_day;
            } else {
                days = ( next.tm_wday + 7 ) - g.energy_reset_day;
            }
            next.tm_mday += days;
            break;
        case RESET_MONTHLY:
            if(g.energy_reset_day == next.tm_mday) {
                next.tm_mon += 1;
            } else {
                next.tm_mday = g.energy_reset_day;
            }
            break;
        }
        mktime(&next);
        g.energy_reset_month = next.tm_mon;
        g.energy_reset_day = next.tm_mday;
        break;
    case RREQ_TEST: // test status only
        log_print(g.log,0,"Status is %d\n",status);

    default:
        break;
    }
    return(status);
}
