//
//        Source: sensor_poll_id.c
//       Created: Tue Dec 20 16:02:25 2022
//            By: Keith Edwin Smith
//     Copyright: (c)2022 Keith Edwin Smith
//         Title: powermeter: poll a specific sensor
//
// Last Modified: 2023-01-07 10:46:15
//

#include "panelmonitor.h"

//----------------------------------------------------------------------
// SENSOR_POLL_ID
// Poll sensor sensor_id for data
//----------------------------------------------------------------------
int sensor_poll_id(modbus_t *psensor, int sensor_id, char *stamp) {
    int x,i;

    if(g.verbose) {
        fprintf(stdout,"Polling: %d\n",sensor_id);
    }
    log_print(g.log,6,"sensor_poll(...,%d,%s)\n",sensor_id,stamp);
    sensor_open(psensor,sensor_id);
    x = modbus_read_input_registers(psensor, 0, 10, buf);
    if (x == -1) {
        log_print(g.log,0,"%s\n", modbus_strerror(errno));
        return -1;
    }

    //for (i=0; i < x; i++) {
    //    printf("reg[%d]=%d (0x%X)\n", i, buf[i], buf[i]);
    //}

    sv.sensor_id = sensor_id;
    strcpy(sv.stamp,stamp);
    sv.volts = buf[REG_VOLTAGE] / 10.0;
    sv.amps =   ((int)(buf[REG_CURRENT_HIGH] << 16) + buf[REG_CURRENT_LOW]) / 1000.0;
    sv.watts =  ((int)(buf[REG_POWER_HIGH] << 16)   + buf[REG_POWER_LOW])   / 10.0;
    sv.energy = ((int)(buf[REG_ENERGY_HIGH] << 16)  + buf[REG_ENERGY_LOW]);
    sv.frequency = buf[REG_FREQUENCY] / 10.0;
    sv.power_factor = buf[REG_POWER_FACTOR] / 100.0;
    sv.alarm = buf[REG_ALARM] > 0 ? 1 : 0;

    modbus_close(psensor);
}
