//
//        Source: sensor_reset_energy.c
//       Created: Tue Dec 20 16:02:25 2022
//            By: Keith Edwin Smith
//     Copyright: (c)2022 Keith Edwin Smith
//         Title: powermeter: Reset the energy accumuator
//
// Last Modified: 2023-01-03 10:50:34
//

#include "panelmonitor.h"

//----------------------------------------------------------------------
// SENSOR_RESET_ENERGY
//----------------------------------------------------------------------
int sensor_reset_energy(modbus_t *psensor, int sensor_id) {
    unsigned char request[8];
    int length,ix;
    uint8_t rsp[MODBUS_TCP_MAX_ADU_LENGTH];

    // Allows for a call to the routine without a sensor number
    if(sensor_id == 0) {
        return(0);
    }

    if(g.verbose) {
        fprintf(stderr,"Resetting Energy Counter\n");
    }

    sensor_open(psensor,sensor_id);
    request[0] = sensor_id;
    request[1] = 0x42;
    length = modbus_send_raw_request(psensor, request, 2);
    modbus_receive_confirmation(psensor, rsp);
    log_print(g.log,1,"RESET: Energy Counter returns: %d\n",length);
    if(g.verbose) {
        fprintf(stderr,"request returns = %d\n",length);
        // fprintf(stderr,"length = %d Respose:\n",length);
        // for(ix = 0; ix < MODBUS_TCP_MAX_ADU_LENGTH; ix++) {
        //     fprintf(stderr,"%02x ",rsp[ix]);
        // }
        // fprintf(stderr,"\n");
    }
    g.energy_reset_success = ENERGY_RESET_PENDING;
    modbus_close(psensor);
}
