//
//        Source: sensor_set_id.c
//       Created: Tue Dec 20 16:02:25 2022
//            By: Keith Edwin Smith
//     Copyright: (c)2022 Keith Edwin Smith
//         Title: powermeter: Set sensor id routine
//
// Last Modified: 2023-01-08 16:02:08
//

#include "panelmonitor.h"

//----------------------------------------------------------------------
// SENSOR_SET_ID
// This should probably wait a few seconds and poll the new id
//----------------------------------------------------------------------
int sensor_set_id(modbus_t *psensor, int current_id, int new_id) {
    unsigned char request[8];
    int length,ix;
    uint8_t rsp[MODBUS_TCP_MAX_ADU_LENGTH];

    // Allows for a call to the routine without a sensor number
    if(g.verbose) {
        log_print(g.log,1,"Sensor ID %d to %d\n",current_id, new_id);
    }

    memset(&rsp,0,MODBUS_TCP_MAX_ADU_LENGTH);
    sensor_open(psensor,current_id);
    request[0] = current_id;
    request[1] = 0x06;
    request[2] = 0x00;
    request[3] = 0x02; // Command 2, RTU Address
    request[4] = 0x00;
    request[5] = new_id;
    length = modbus_send_raw_request(psensor, request, 6);
    modbus_receive_confirmation(psensor, rsp);
    log_print(g.log,0,"SETID: length: %d Response:\n",length);
    for(ix = 0; ix < length; ix++) {
        log_print(g.log,0,"%02x ",rsp[ix]);
    }
    log_print(g.log,0,"\n");
    modbus_close(psensor);
}
