//
//        Source: show_config.c
//       Created: Tue Dec 20 16:02:25 2022
//            By: Keith Edwin Smith
//     Copyright: (c)2022 Keith Edwin Smith
//         Title: powermeter: Dump the config items for the current invocation
//
// Last Modified: 2023-01-08 08:28:34
//
#include "panelmonitor.h"

int show_config(LOG_STRUCT *log, int level) {
    int ix;
    HIDRELAY *hr;
    RELAY *rs;

    if(log == NULL) {
        fprintf(stderr,"\n");
        fprintf(stderr,"Panel Monitor Status\n");
        fprintf(stderr,"\n");
        fprintf(stderr,"RS-485 device: %s\n",g.device_path);
        fprintf(stderr,"Reset Energy: %s  on day %d\n",g.energy_reset,g.energy_reset_day);
        fprintf(stderr,"Polling Interval: %d\n",g.interval);
        fprintf(stderr,"Sensors:");
        for(ix = 0; g.sensor_list[ix] != 0; ix++) {
            fprintf(stderr," %d",g.sensor_list[ix]);
        }
        fprintf(stderr,"\n");
        fprintf(stderr,"Relays:\n");
        fprintf(stderr,"  Minimum Off Time: %d\n",g.relay_min_off_time);
        for(hr = g.hr_first; hr != NULL; hr = hr->next) {
            for(ix = 0; ix < MAX_RELAYS; ix++) {
                if(g.relay[ix] != NULL) {
                    rs = g.relay[ix];
                    fprintf(stderr,"    %2d/%-3s %-30s ACT/OVR: %d/%02x ON/OFF: %d/%d\n"
                            ,ix+1
                            ,hidrelay_status(hr->state[ix])
                            ,rs->label
                            ,rs->activate
                            ,rs->override
                            ,rs->threshold_on
                            ,rs->threshold_off);
                }
            }
        }
    } else {
        log_print(log,level,"dev:%s",g.device_path);
        log_print(log,level," reset:%s/%d",g.energy_reset,g.energy_reset_day);
        log_print(log,level," poll:%dm",g.interval);
        for(ix = 0; g.sensor_list[ix] != 0; ix++) {
            log_print(log,level," %d",g.sensor_list[ix]);
        }
        log_print(log,level,"\n");
    }
}

int log_relay_status(LOG_STRUCT *log,int level) {
    HIDRELAY *hr;
    RELAY *rs;
    int ix;
    // log_print(log,level,"  Minimum Off Time: %d\n",g.relay_min_off_time);
    for(hr = g.hr_first; hr != NULL; hr = hr->next) {
        for(ix = 0; ix < MAX_RELAYS; ix++) {
            if(g.relay[ix] != NULL) {
                rs = g.relay[ix];
                log_print(log,level,"RELAY: %2d/%-3s %-30s ACT/OVR: %s/%02x ON/OFF: %d/%d\n"
                        ,ix+1
                        ,hidrelay_status(hr->state[ix])
                        ,rs->label
                        ,relay_activate_words[rs->activate]
                        ,rs->override
                        ,rs->threshold_on
                        ,rs->threshold_off);
            }
        }
    }
}
